@@ -16,8 +16,8 @@ See homepage for examples of usage: <http://github.com/lymar/hastache>
 Simplest example:
 
 @
-import Text.Hastache 
-import Text.Hastache.Context 
+import           Text.Hastache
+import           Text.Hastache.Context
 import qualified Data.Text.Lazy.IO as TL
 
 main = do 
@@ -41,22 +41,23 @@ You have 100 unread messages.
 Using Generics:
 
 @
-import Text.Hastache 
-import Text.Hastache.Context 
+{&#45;\# LANGUAGE DeriveDataTypeable, OverloadedStrings \#&#45;}
+
+import           Text.Hastache
+import           Text.Hastache.Context
 import qualified Data.Text.Lazy.IO as TL
-import Data.Data 
-import Data.Generics 
- 
-data Info = Info { 
-    name    :: String, 
-    unread  :: Int 
+import           Data.Data
+
+data Info = Info {
+    name    :: String,
+    unread  :: Int
     } deriving (Data, Typeable)
 
-main = do 
-    res <- hastacheStr defaultConfig (encodeStr template) 
-        (mkGenericContext inf) 
-    TL.putStrLn res 
-  where 
+main = do
+    res <- hastacheStr defaultConfig template
+        (mkGenericContext inf)
+    TL.putStrLn res
+  where
     template = \"Hello, {{name}}!\\n\\nYou have {{unread}} unread messages.\"
     inf = Info \"Haskell\" 100
 @
@@ -71,6 +72,7 @@ module Text.Hastache (
     , MuType(..)
     , MuConfig(..)
     , MuVar(..)
+    , composeCtx
     , htmlEscape
     , emptyEscape
     , defaultConfig
@@ -86,12 +88,13 @@ import Control.Monad.Trans (lift, liftIO, MonadIO)
 import Control.Monad.Trans.Maybe (MaybeT(MaybeT), runMaybeT)
 import Data.AEq (AEq,(~==))
 import Data.Functor ((<$>))
-import Data.Int
 import Data.IORef
+import Data.Int
 import Data.Maybe (isJust)
-import Data.Monoid (mappend, mempty)
+import Data.Monoid (Monoid, mappend, mempty)
 import Data.Text hiding (map, foldl1)
 import Data.Text.IO
+import Data.Version (Version)
 import Data.Word
 import Prelude hiding (putStrLn, readFile, length, drop, tail, dropWhile, elem,
     head, last, reverse, take, span, null)
@@ -119,8 +122,24 @@ type MuContext m =
     Text            -- ^ Variable name
     -> m (MuType m) -- ^ Value
 
+instance (Monad m) => Monoid (MuContext m) where
+    mempty = const $ return MuNothing
+    a `mappend` b = \v -> do
+        x <- a v
+        case x of
+            MuNothing -> b v
+            _ -> return x
+
+-- | Left-leaning compoistion of contexts. Given contexts @c1@ and
+-- @c2@, the behaviour of @(c1 <> c2) x@ is following: if @c1 x@
+-- produces 'MuNothing', then the result is @c2 x@. Otherwise the
+-- result is @c1 x@. Even if @c1 x@ is 'MuNothing', the monadic
+-- effects of @c1@ are still to take place.
+composeCtx :: (Monad m) => MuContext m -> MuContext m -> MuContext m
+composeCtx = mappend
+
 class Show a => MuVar a where
-    -- | Convert to Lazy ByteString
+    -- | Convert to lazy 'Data.Text.Lazy.Text'
     toLText   :: a -> TL.Text
     -- | Is empty variable (empty string, zero number etc.)
     isEmpty   :: a -> Bool 
@@ -163,7 +182,8 @@ instance MuVar Word32  where {toLText = withShowToText; isEmpty = numEmpty}
 instance MuVar Word64  where {toLText = withShowToText; isEmpty = numEmpty}
 instance MuVar ()      where {toLText = withShowToText}
 
-    
+instance MuVar Version where {toLText = withShowToText }
+
 instance MuVar Char where
     toLText = TL.singleton
 
@@ -212,7 +232,7 @@ data MuConfig m = MuConfig {
     muEscapeFunc        :: TL.Text -> TL.Text, 
         -- ^ Escape function ('htmlEscape', 'emptyEscape' etc.)
     muTemplateFileDir   :: Maybe FilePath,
-        -- ^ Directory for search partial templates ({{> templateName}})
+        -- ^ Directory for search partial templates (@{{> templateName}}@)
     muTemplateFileExt   :: Maybe String,
         -- ^ Partial template files extension
     muTemplateRead      :: FilePath -> m (Maybe Text)
@@ -220,22 +240,23 @@ data MuConfig m = MuConfig {
         --   template could not be found.
     }
 
--- | Convert String to Text
+-- | Convert 'String' to 'Text'
 encodeStr :: String -> Text
 encodeStr = T.pack
 
--- | Convert String to Lazy Text
+-- | Convert 'String' to Lazy 'Data.Text.Lazy.Text'
 encodeStrLT :: String -> TL.Text
 encodeStrLT = TL.pack
 
--- | Convert Text to String
+-- | Convert 'Text' to 'String'
 decodeStr :: Text -> String
 decodeStr = T.unpack
 
--- | Convert Lazy Text to String
+-- | Convert Lazy 'Data.Text.Lazy.Text' to 'String'
 decodeStrLT :: TL.Text -> String
 decodeStrLT = TL.unpack
 
+-- | isMuNothing x = x == MuNothing
 isMuNothing :: MuType t -> Bool
 isMuNothing MuNothing = True
 isMuNothing _ = False
@@ -352,7 +373,7 @@ findCloseSection str name otag ctag = do
     (before, after) = breakOn close str
 
 trimCharsTest :: Char -> Bool
-trimCharsTest = (`Prelude.elem` " \t")
+trimCharsTest = (`Prelude.elem` [' ', '\t'])
 
 trimAll :: Text -> Text
 trimAll = dropAround trimCharsTest
