@@ -1,5 +1,6 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE RankNTypes #-}
 -- Module:      Text.Hastache.Context
 -- Copyright:   Sergey S Lymar (c) 2011-2013 
 -- License:     BSD3
@@ -14,11 +15,16 @@ module Text.Hastache.Context (
       mkStrContext
     , mkStrContextM
     , mkGenericContext
+    , mkGenericContext'
+    , Ext
+    , defaultExt
     ) where 
 
 import Data.Data
 import Data.Generics
 import Data.Int
+import Data.Version (Version)
+import Data.Ratio (Ratio)
 import Data.Word
 
 import qualified Data.ByteString as BS
@@ -41,11 +47,19 @@ mkStrContext f a = decodeStr a ~> f ~> return
 mkStrContextM :: Monad m => (String -> m (MuType m)) -> MuContext m
 mkStrContextM f a = decodeStr a ~> f
 
+type Ext = forall b. (Data b, Typeable b) => b -> String
+
+-- | @defaultExt ==@ 'gshow'
+defaultExt :: Ext
+defaultExt = gshow
+
 {- | 
 Make Hastache context from Data.Data deriving type
 
 Supported field types:
 
+ * ()
+
  * String
  
  * Char
@@ -86,6 +100,12 @@ Supported field types:
  
  * Bool
 
+ * Version
+
+ * Maybe @a@ (where @a@ is a supported datatype)
+
+ * Either @a@ @b@ (where @a@ and @b@ are supported datatypes)
+
  * Data.Text.Text -> Data.Text.Text
 
  * Data.Text.Text -> Data.Text.Lazy.Text
@@ -142,7 +162,7 @@ data Example = Example {
 example = hastacheStr defaultConfig (encodeStr template) 
     (mkGenericContext context)
     where
-    template = concat $ map (++ \"\\n\") [
+    template = unlines [
         \"string: {{stringField}}\",
         \"int: {{intField}}\",
         \"data: {{dataField.someField}}, {{dataField.anotherField}}\",
@@ -199,14 +219,79 @@ B : {{num}}
 @
 
 -}
-                    
 #if MIN_VERSION_base(4,7,0)
 mkGenericContext :: (Monad m, Data a, Typeable m) => a -> MuContext m
 #else
 mkGenericContext :: (Monad m, Data a, Typeable1 m) => a -> MuContext m
 #endif
-mkGenericContext val = toGenTemp val ~> convertGenTempToContext
-    
+mkGenericContext val = toGenTemp id defaultExt val ~> convertGenTempToContext
+
+{-| 
+
+Like 'mkGenericContext', but apply the first function to record field
+names when constructing the context. The second function is used to
+constructing values for context from datatypes that are nor supported
+as primitives in the library. The resulting value can be accessed
+using the @.DatatypeName@ field:
+
+@
+\{\-\# LANGUAGE DeriveDataTypeable \#\-\}
+\{\-\# LANGUAGE FlexibleInstances \#\-\}
+\{\-\# LANGUAGE ScopedTypeVariables \#\-\}
+\{\-\# LANGUAGE StandaloneDeriving \#\-\}
+\{\-\# LANGUAGE TypeSynonymInstances \#\-\}
+import Text.Hastache 
+import Text.Hastache.Context 
+
+import qualified Data.Text.Lazy as TL 
+import qualified Data.Text.Lazy.IO as TL 
+
+import Data.Data (Data, Typeable)
+import Data.Decimal
+import Data.Generics.Aliases (extQ)
+
+data Test = Test {n::Int, m::Decimal} deriving (Data, Typeable)
+deriving instance Data Decimal
+
+val :: Test
+val = Test 1 (Decimal 3 1500)
+
+q :: Ext
+q = defaultExt \`extQ\` (\(i::Decimal) -> "A decimal: " ++ show i)
+
+r "m" = "moo"
+r x   = x
+
+example :: IO TL.Text
+example = hastacheStr defaultConfig
+                      (encodeStr template)
+                      (mkGenericContext' r q val)
+
+template = concat [ 
+     "{{n}}\\n",
+     "{{moo.Decimal}}"
+     ] 
+
+main = example >>= TL.putStrLn
+@
+
+Result:
+
+@
+1
+A decimal: 1.500
+@
+
+-}
+#if MIN_VERSION_base(4,7,0)
+mkGenericContext' :: (Monad m, Data a, Typeable m) 
+                  => (String -> String) -> Ext -> a -> MuContext m
+#else
+mkGenericContext' :: (Monad m, Data a, Typeable1 m) 
+                  => (String -> String) -> Ext -> a -> MuContext m
+#endif
+mkGenericContext' f ext val = toGenTemp f ext val ~> convertGenTempToContext
+
 data TD m = 
       TSimple (MuType m) 
     | TObj [(String, TD m)] 
@@ -215,62 +300,80 @@ data TD m =
     deriving (Show)
 
 #if MIN_VERSION_base(4,7,0)
-toGenTemp :: (Data a, Monad m, Typeable m) => a -> TD m
+toGenTemp :: (Data a, Monad m, Typeable m) 
+          => (String -> String) -> Ext -> a -> TD m
 #else
-toGenTemp :: (Data a, Monad m, Typeable1 m) => a -> TD m
+toGenTemp :: (Data a, Monad m, Typeable1 m) 
+          => (String -> String) -> Ext -> a -> TD m
 #endif
-toGenTemp a = TObj $ conName : zip fields (gmapQ procField a)
+toGenTemp f g a = TObj $ conName : zip fields (gmapQ (procField f g) a) 
     where
-    fields = toConstr a ~> constrFields
-    conName = (toConstr a ~> showConstr, MuBool True ~> TSimple)
+    fields = toConstr a ~> constrFields ~> map f
+    conName = (toConstr a ~> showConstr, TSimple . MuVariable $ g a)
 
 #if MIN_VERSION_base(4,7,0)
-procField :: (Data a, Monad m, Typeable m) => a -> TD m
+procField :: (Data a, Monad m, Typeable m) 
+          => (String -> String) -> Ext -> a -> TD m
 #else
-procField :: (Data a, Monad m, Typeable1 m) => a -> TD m
+procField :: (Data a, Monad m, Typeable1 m) 
+          => (String -> String) -> Ext -> a -> TD m
 #endif
-procField = 
-    obj
-    `ext1Q` list
-    `extQ` (\(i::String)            -> MuVariable (encodeStr i) ~> TSimple)
-    `extQ` (\(i::Char)              -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Double)            -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Float)             -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Int)               -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Int8)              -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Int16)             -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Int32)             -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Int64)             -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Integer)           -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Word)              -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Word8)             -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Word16)            -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Word32)            -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Word64)            -> MuVariable i ~> TSimple)
-    `extQ` (\(i::BS.ByteString)     -> MuVariable i ~> TSimple)
-    `extQ` (\(i::LBS.ByteString)    -> MuVariable i ~> TSimple)
-    `extQ` (\(i::T.Text)            -> MuVariable i ~> TSimple)
-    `extQ` (\(i::TL.Text)           -> MuVariable i ~> TSimple)
-    `extQ` (\(i::Bool)              -> MuBool i     ~> TSimple)
+procField f g a = 
+    case res a of
+      TUnknown -> TSimple . MuVariable . g $ a          
+      b        -> b
+  where
+    res = obj
+        `ext1Q` list
+        `extQ` (\(i::String)            -> MuVariable (encodeStr i) ~> TSimple)
+        `extQ` (\(i::Char)              -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Double)            -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Float)             -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Int)               -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Int8)              -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Int16)             -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Int32)             -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Int64)             -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Integer)           -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Word)              -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Word8)             -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Word16)            -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Word32)            -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Word64)            -> MuVariable i ~> TSimple)
+        `extQ` (\(i::BS.ByteString)     -> MuVariable i ~> TSimple)
+        `extQ` (\(i::LBS.ByteString)    -> MuVariable i ~> TSimple)
+        `extQ` (\(i::T.Text)            -> MuVariable i ~> TSimple)
+        `extQ` (\(i::TL.Text)           -> MuVariable i ~> TSimple)
+        `extQ` (\(i::Bool)              -> MuBool i     ~> TSimple)
+        `extQ` (\()                     -> MuVariable () ~> TSimple)
+        `extQ` (\(i::Version)           -> MuVariable i ~> TSimple)
+        `extQ` muLambdaTT
+        `extQ` muLambdaTTL
+        `extQ` muLambdaTLTL
+        `extQ` muLambdaBSBS
+        `extQ` muLambdaSS
+        `extQ` muLambdaBSLBS
+
+        `extQ` muLambdaMTT
+        `extQ` muLambdaMTTL
+        `extQ` muLambdaMTLTL
+        `extQ` muLambdaMBSBS
+        `extQ` muLambdaMSS
+        `extQ` muLambdaMBSLBS
+
+        `ext1Q` muMaybe
+        `ext2Q` muEither 
 
-    `extQ` muLambdaTT
-    `extQ` muLambdaTTL
-    `extQ` muLambdaTLTL
-    `extQ` muLambdaBSBS
-    `extQ` muLambdaSS
-    `extQ` muLambdaBSLBS
-    
-    `extQ` muLambdaMTT
-    `extQ` muLambdaMTTL
-    `extQ` muLambdaMTLTL
-    `extQ` muLambdaMBSBS
-    `extQ` muLambdaMSS
-    `extQ` muLambdaMBSLBS
-    where
     obj a = case dataTypeRep (dataTypeOf a) of
-        AlgRep (_:_) -> toGenTemp a
+        AlgRep (_:_) -> toGenTemp f g a
         _ -> TUnknown
-    list a = map procField a ~> TList
+    list a = map (procField f g) a ~> TList
+
+    muMaybe Nothing = TSimple MuNothing
+    muMaybe (Just a) = TList [procField f g a]
+
+    muEither (Left a) = procField f g a
+    muEither (Right b) = procField f g b
 
     muLambdaTT :: (T.Text -> T.Text) -> TD m
     muLambdaTT f = MuLambda f ~> TSimple
@@ -302,7 +405,7 @@ procField =
 
     muLambdaMTTL :: (T.Text -> m TL.Text) -> TD m
     muLambdaMTTL f = MuLambdaM f ~> TSimple
-        
+
     muLambdaMBSBS :: (BS.ByteString -> m BS.ByteString) -> TD m
     muLambdaMBSBS f = MuLambdaM (f . T.encodeUtf8) ~> TSimple
 
